@@ -1,8 +1,10 @@
-maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
-                    fixed, removeDuplicates=FALSE, na.action="na.omit", ...)
+maxlike <- function(formula, rasters, points, link=c("logit", "cloglog"),
+                    starts, hessian=TRUE, fixed, removeDuplicates=FALSE,
+                    savedata=FALSE, na.action="na.omit", ...)
 {
     if(identical(formula, ~1))
         stop("At least one continuous covariate must be specified in the formula")
+    link <- match.arg(link)
     varnames <- all.vars(formula)
     call <- match.call()
     npts <- nrow(points)
@@ -16,30 +18,34 @@ maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
         stop("points must have 2 columns containing the x- and y- coordinates")
     pt.names <- colnames(points)
     if(identical(cd.class, "RasterStack")) {
-        cd.names <- layerNames(rasters)
+        cd.names <- names(rasters)
         npix <- prod(dim(rasters)[1:2])
         cellID <- cellFromXY(rasters, points)
         duplicates <- duplicated(cellID)
         if(removeDuplicates) {
             cellID <- unique(cellID)
             npts <- length(cellID)
+            points.retained <- points[!duplicates,]
             }
         x <- as.data.frame(matrix(extract(rasters, cellID), npts))
         z <- as.data.frame(matrix(getValues(rasters), npix))
         names(x) <- names(z) <- cd.names
         }
     if(!all(varnames %in% cd.names))
-        stop("at least 1 covariate in the formula is not in rasters.")
+        stop("at least 1 covariate in the formula is not in layerNames(rasters).")
     X.mf <- model.frame(formula, x, na.action=na.action)
     X.mf.a <- attributes(X.mf)
     pts.removed <- integer(0)
+    points.retained <- points
     if("na.action" %in% names(X.mf.a)) {
         pts.removed <- X.mf.a$na.action
         npts.removed <- length(pts.removed)
-        if(npts.removed > 0)
+        if(npts.removed > 0) {
             warning(paste(npts.removed,
                           "points removed due to missing values"))
+            points.retained <- points.retained[-pts.removed,]
         }
+    }
     X <- model.matrix(formula, X.mf)
     Z.mf <- model.frame(formula, z, na.action=na.action)
     Z.mf.a <- attributes(Z.mf)
@@ -47,9 +53,9 @@ maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
     if("na.action" %in% names(Z.mf.a)) {
         pix.removed <- Z.mf.a$na.action
         npix.removed <- length(pix.removed)
-        if(npix.removed > 0)
-            warning(paste(npix.removed,
-                          "pixels removed due to missing values"))
+#        if(npix.removed > 0)
+#            warning(paste(npix.removed,
+#                          "pixels removed due to missing values"))
         }
     Z <- model.matrix(formula, Z.mf)
     npars <- ncol(X)
@@ -63,11 +69,20 @@ maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
     else
        names(starts) <- parnames
 
-    nll <- function(pars) {
-        psix <- plogis(X %*% pars)
-        psiz <- plogis(Z %*% pars)
-        -1*sum(log(psix/sum(psiz) + .Machine$double.eps))
+    if(identical(link, "logit")) {
+        nll <- function(pars) {
+            psix <- plogis(drop(X %*% pars))
+            psiz <- sum(plogis(drop(Z %*% pars)))
+            -1*sum(log(psix/psiz)) # + .Machine$double.xmin))
         }
+    } else if(identical(link, "cloglog")) {
+        nll <- function(pars) {
+            psix <- 1-exp(-exp(drop(X %*% pars)))
+            psiz <- sum(1-exp(-exp(drop(Z %*% pars))))
+            -1*sum(log(psix/psiz)) # + .Machine$double.xmin))
+        }
+    } else
+        stop("link function should be either 'logit' or 'cloglog'")
 
     is.fixed <- rep(FALSE, npars)
     if(!missing(fixed)) {
@@ -112,8 +127,10 @@ maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
     fitted <- plogis(Z %*% par)
     out <- list(Est=cbind(Est=par, SE=se), vcov=vc, AIC=aic, call=call,
                 pts.removed=pts.removed, pix.removed=pix.removed,
-#                duplicates=duplicates,
-                optim=fm, not.fixed=not.fixed)
+                points.retained=points.retained,
+                optim=fm, not.fixed=not.fixed, link=link)
+    if(savedata)
+        out$rasters <- rasters
     class(out) <- c("maxlikeFit", "list")
     return(out)
     }
